window.Gleam.initSearch([{"doc":"cigogne","title":"cigogne","content":"# 🪽 cigogne - Easy migrations in Gleam\n\n[![Package Version](https://img.shields.io/hexpm/v/cigogne)](https://hex.pm/packages/cigogne)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/cigogne/)\n\nCigogne (French for stork) is a simple tool to manage migrations for your Postgres database.\nThe idea is that instead of writing schemas in Gleam and having a tool do the migration scripts for you,\nwith Cigogne you have total control on your migrations. That approach gives you more freedom on how you want to do\nyour migrations for the price of a bit of time and learning some database skills.\n\nFor example, if you decide to rename a column, you can use a `ALTER TABLE RENAME COLUMN` query.  \nIf you decide to change a column&#39;s data type, you can decide how to populate it based on the old values.\n\nAs a plus, this integrates really well with the approach provided by [squirrel](https://hexdocs.pm/squirrel/).\n\n## Installation\n\n```sh\ngleam add --dev cigogne\n```\n\nCigogne can also be installed as a regular dependency and used in your project. See documentation about the `cigogne` module to know more.\n\n## Usage\n\nTo use `cigogne`, you first have to create your migration scripts in the `priv/migrations` folder.\nCigogne will look for `.sql` files in this folder to get your migrations. Cigogne expects \nthe sql files to have the following format `&lt;MigrationTimestamps&gt;-&lt;MigrationName&gt;.sql`. This way we \ncan get an order in which migrations should be applied and a descriptive name right away. It also\nallows for easy sorting of your migration scripts and forces you to give it a meaningful name.\n\n&gt; This may not be great for everyone, so it may change in the future, maybe to some kind of front matter or something else.  \n&gt; Please let me know what you think 😉\n\nYou can then write your migration scripts. It should look like this \n\n```sql\n--- migration:up\nCREATE TABLE users(\n    id UUID PRIMARY KEY,\n    firstName TEXT NOT NULL,\n    age INT NOT NULL\n);\n\n--- migration:down\nDROP TABLE users;\n\n--- migration:end\n```\n\nThe `--- migration:` comments are used to separate between the queries used for the &quot;up migration&quot; and the &quot;down migration&quot;.\n\n&gt; Again this may change if we find a better way of doing it !\n\nYou are now ready to execute your migration ! The first way of executing your migration is via the command line :\n\n```sh\n# Apply the next migration\ngleam run -m cigogne up\n# Roll back the last applied migration\ngleam run -m cigogne down\n# Apply all migrations not yet applied\ngleam run -m cigogne last\n# Apply/roll back N migrations (apply if N &gt; 0, roll back otherwise)\ngleam run -m cigogne apply N\n# Show the last applied migration and the current DB schema\ngleam run -m cigogne show\n# Create a new migration with name NAME\ngleam run -m cigogne new NAME\n```\n\nYou can also use it in your server initalisation function to make sure your schema is up-to-date with your code.\n\n```gleam\nimport cigogne\n\npub fn init() -&gt; Nil {\n    use db_connection &lt;- db_connect() // to be implemented\n    cigogne.execute_migrations_to_last(db_connection)\n    |&gt; result.then(cigogne.update_schema_file(db_url))\n    |&gt; result.map_error(cigogne.print_error)\n    |&gt; result.unwrap_both\n\n    // Or directly by using environment variables\n    cigogne.migrate_to_last()\n    |&gt; result.map_error(cigogne.print_error)\n    |&gt; result.unwrap_both\n}\n```\n\n&gt; By default, `cigogne` gets the database url from the `DATABASE_URL` environment variable.  \n&gt; Also, the migrate_up, _down, _n, and _to_last functions systematically update a schema file\n&gt; whenever they are successful.\n\nFurther documentation can be found at &lt;https://hexdocs.pm/cigogne&gt;.\n\n## Development ideas\n\nI have a few ideas to improve this library that I may implement in the future.  \nLet me know if some of those are of interest to you and I will prioritize them.\n\n- Specify default schema (for schema file) via an envvar.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"cigogne/types","title":"MigrateError","content":"pub type MigrateError {\n  EnvVarError(name: String)\n  UrlError(url: String)\n  FileError(path: String)\n  PatternError(error: String)\n  FileNameError(path: String)\n  CompoundError(errors: List(MigrateError))\n  ContentError(path: String, error: String)\n  PGOTransactionError(error: pog.TransactionError)\n  PGOQueryError(error: pog.QueryError)\n  NoResultError\n  SchemaQueryError(error: String)\n  NoMigrationToApplyError\n  NoMigrationToRollbackError\n  MigrationNotFoundError(\n    timestamp: tempo.NaiveDateTime,\n    name: String,\n  )\n  DateParseError(date: String)\n}\n The errors returned by cigogne\nEnvVarError(name: String)\n\n\nUrlError(url: String)\n\n\nFileError(path: String)\n\n\nPatternError(error: String)\n\n\nFileNameError(path: String)\n\n\nCompoundError(errors: List(MigrateError))\n\n\nContentError(path: String, error: String)\n\n\nPGOTransactionError(error: pog.TransactionError)\n\n\nPGOQueryError(error: pog.QueryError)\n\n\nNoResultError\n\n\nSchemaQueryError(error: String)\n\n\nNoMigrationToApplyError\n\n\nNoMigrationToRollbackError\n\n\nMigrationNotFoundError(\n  timestamp: tempo.NaiveDateTime,\n  name: String,\n)\n\n\nDateParseError(date: String)\n\n\nSynonyms:\ncigogne/types.MigrateError\ncigogne/types MigrateError","url":"cigogne/types.html#MigrateError"},{"doc":"cigogne/types","title":"Migration","content":"pub type Migration {\n  Migration(\n    path: String,\n    timestamp: tempo.NaiveDateTime,\n    name: String,\n    queries_up: List(String),\n    queries_down: List(String),\n  )\n}\n Migrations are often generated by reading migration files.  \n However, we allow you to create your own Migrations\nMigration(\n  path: String,\n  timestamp: tempo.NaiveDateTime,\n  name: String,\n  queries_up: List(String),\n  queries_down: List(String),\n)\n\n\nSynonyms:\ncigogne/types.Migration\ncigogne/types Migration","url":"cigogne/types.html#Migration"},{"doc":"cigogne/types","title":"print_migrate_error","content":"pub fn print_migrate_error(error: MigrateError) -&gt; Nil\n Print a MigrateError to the stderr\nSynonyms:\ncigogne/types.print_migrate_error\ncigogne/types print_migrate_error","url":"cigogne/types.html#print_migrate_error"},{"doc":"cigogne/types","title":"cigogne/types","content":"","url":"cigogne/types.html"},{"doc":"cigogne","title":"apply_migration","content":"pub fn apply_migration(\n  connection: Connection,\n  migration: Migration,\n) -&gt; Result(Nil, MigrateError)\n Apply a migration to the database.  \n This function does not create a schema file.\nSynonyms:\ncigogne.apply_migration\ncigogne apply_migration","url":"cigogne.html#apply_migration"},{"doc":"cigogne","title":"apply_next_migration","content":"pub fn apply_next_migration(\n  connection: Connection,\n) -&gt; Result(Nil, MigrateError)\n Apply the next migration that wasn&#39;t applied yet.  \n The migrations are acquired from priv/migrations/*.sql files.  \n This function does not create a schema file.\nSynonyms:\ncigogne.apply_next_migration\ncigogne apply_next_migration","url":"cigogne.html#apply_next_migration"},{"doc":"cigogne","title":"execute_migrations_to_last","content":"pub fn execute_migrations_to_last(\n  connection: Connection,\n) -&gt; Result(Nil, MigrateError)\n Apply migrations until we reach the last defined migration.  \n The migrations are acquired from **/migrations/*.sql files.  \n This function does not create a schema file.\nSynonyms:\ncigogne.execute_migrations_to_last\ncigogne execute_migrations_to_last","url":"cigogne.html#execute_migrations_to_last"},{"doc":"cigogne","title":"execute_n_migrations","content":"pub fn execute_n_migrations(\n  connection: Connection,\n  count: Int,\n) -&gt; Result(Nil, MigrateError)\n Apply or roll back migrations until we reach the migration corresponding to the provided number.  \n The migrations are acquired from **/migrations/*.sql files.  \n This function does not create a schema file.\nSynonyms:\ncigogne.execute_n_migrations\ncigogne execute_n_migrations","url":"cigogne.html#execute_n_migrations"},{"doc":"cigogne","title":"get_migrations","content":"pub fn get_migrations() -&gt; Result(List(Migration), MigrateError)\n Get all defined migrations in your project.  \n Migration files are searched in `/migrations` folders.\nSynonyms:\ncigogne.get_migrations\ncigogne get_migrations","url":"cigogne.html#get_migrations"},{"doc":"cigogne","title":"get_schema","content":"pub fn get_schema(url: String) -&gt; Result(String, MigrateError)\n Get details about the schema of the database at the provided url.\nSynonyms:\ncigogne.get_schema\ncigogne get_schema","url":"cigogne.html#get_schema"},{"doc":"cigogne","title":"main","content":"pub fn main() -&gt; Result(Nil, Nil)\n\nSynonyms:\ncigogne.main\ncigogne main","url":"cigogne.html#main"},{"doc":"cigogne","title":"migrate_down","content":"pub fn migrate_down() -&gt; Result(Nil, MigrateError)\n Roll back the last applied migration.  \n This function will get the database url from the `DATABASE_URL` environment variable.  \n The migrations are then acquired from priv/migrations/*.sql files.  \n If successful, it will also create a file and write details of the new schema in it.\nSynonyms:\ncigogne.migrate_down\ncigogne migrate_down","url":"cigogne.html#migrate_down"},{"doc":"cigogne","title":"migrate_n","content":"pub fn migrate_n(count: Int) -&gt; Result(Nil, MigrateError)\n Apply or roll back migrations until we reach the migration corresponding to the provided number.  \n This function will get the database url from the `DATABASE_URL` environment variable.  \n The migrations are then acquired from priv/migrations/*.sql files.  \n If successful, it will also create a file and write details of the new schema in it.\nSynonyms:\ncigogne.migrate_n\ncigogne migrate_n","url":"cigogne.html#migrate_n"},{"doc":"cigogne","title":"migrate_to_last","content":"pub fn migrate_to_last() -&gt; Result(Nil, MigrateError)\n Apply migrations until we reach the last defined migration.  \n This function will get the database url from the `DATABASE_URL` environment variable.  \n The migrations are then acquired from priv/migrations/*.sql files.  \n If successful, it will also create a file and write details of the new schema in it.\nSynonyms:\ncigogne.migrate_to_last\ncigogne migrate_to_last","url":"cigogne.html#migrate_to_last"},{"doc":"cigogne","title":"migrate_up","content":"pub fn migrate_up() -&gt; Result(Nil, MigrateError)\n Apply the next migration that wasn&#39;t applied yet.  \n This function will get the database url from the `DATABASE_URL` environment variable.  \n The migrations are then acquired from priv/migrations/*.sql files.  \n If successful, it will also create a file and write details of the new schema in it.\nSynonyms:\ncigogne.migrate_up\ncigogne migrate_up","url":"cigogne.html#migrate_up"},{"doc":"cigogne","title":"new_migration","content":"pub fn new_migration(name: String) -&gt; Result(Nil, MigrateError)\n Create a new migration file in the `priv/migrations` folder with the provided name.\nSynonyms:\ncigogne.new_migration\ncigogne new_migration","url":"cigogne.html#new_migration"},{"doc":"cigogne","title":"print_error","content":"pub fn print_error(error: MigrateError) -&gt; Nil\n Print a MigrateError to the standard error stream.\nSynonyms:\ncigogne.print_error\ncigogne print_error","url":"cigogne.html#print_error"},{"doc":"cigogne","title":"roll_back_migration","content":"pub fn roll_back_migration(\n  connection: Connection,\n  migration: Migration,\n) -&gt; Result(Nil, MigrateError)\n Roll back a migration from the database.  \n This function does not create a schema file.\nSynonyms:\ncigogne.roll_back_migration\ncigogne roll_back_migration","url":"cigogne.html#roll_back_migration"},{"doc":"cigogne","title":"roll_back_previous_migration","content":"pub fn roll_back_previous_migration(\n  connection: Connection,\n) -&gt; Result(Nil, MigrateError)\n Roll back the last applied migration.  \n The migrations are acquired from **/migrations/*.sql files.  \n This function does not create a schema file.\nSynonyms:\ncigogne.roll_back_previous_migration\ncigogne roll_back_previous_migration","url":"cigogne.html#roll_back_previous_migration"},{"doc":"cigogne","title":"update_schema_file","content":"pub fn update_schema_file(\n  url: String,\n) -&gt; Result(Nil, MigrateError)\n Create or update a schema file with details of the schema of the database at the provided url.\n The schema file is created at `./sql.schema`.\nSynonyms:\ncigogne.update_schema_file\ncigogne update_schema_file","url":"cigogne.html#update_schema_file"},{"doc":"cigogne","title":"cigogne","content":"","url":"cigogne.html"}]);